<?php

namespace FiloBlu\CegidY2\Service;

use Magento\Framework\App\Config\ScopeConfigInterface;
use WsdlToPhp\PackageBase\SoapClientInterface;
use RuntimeException;

/**
 * Class CredentialProvider
 * @package FiloBlu\CegidY2Api\Service
 */
class CredentialProvider
{
    /**
     * @var string
     */
    const PATH_FORMAT_STRING = 'filoblu_cegid_y2/%s/%s';
    /**
     * @var string
     */
    const URL = 'url';
    /**
     * @var string
     */
    const LOGIN = 'login';
    /**
     * @var string
     */
    const PASSWORD = 'password';
    /**
     * @var string
     */
    const DATABASE = 'database';
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var array
     */
    protected $configurations = [];
    /**
     * @var array
     */
    protected $serviceMap;

    /**
     * CredentialProvider constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param array $serviceMap
     */
    public function __construct(ScopeConfigInterface $scopeConfig, $serviceMap = [])
    {
        $this->scopeConfig = $scopeConfig;
        $this->serviceMap = $serviceMap;
    }

    /**
     * @return array
     */
    public function getServiceMap()
    {
        return $this->serviceMap;
    }

    /**
     * @param $serviceId
     * @return array
     */
    public function getCredentials($serviceId)
    {
        $options = [];
        foreach ([self::URL, self::LOGIN, self::PASSWORD] as $key) {

            if ($value = $this->getConfigurationValue($serviceId, $key)){
                $options[SoapClientInterface::OPTION_PREFIX . $key] = $value;
            } else {
                throw new RuntimeException("Empty Cegid getCredentials WSDL {$key}");
            }

        }

        return $options;
    }

    /**
     * @param $serviceId
     * @param $key
     * @return mixed
     */
    public function getConfigurationValue($serviceId, $key)
    {
        if (isset($this->configurations[$serviceId][$key])) {
            return $this->configurations[$serviceId][$key];
        }

        if (!isset($this->configurations['default'][$key])) {
            $this->configurations['default'][$key] =
                $this->scopeConfig->getValue(sprintf(self::PATH_FORMAT_STRING, 'default', $key));
        }

        $value = $this->scopeConfig->getValue(sprintf(self::PATH_FORMAT_STRING, $serviceId, $key));

        $this->configurations[$serviceId][$key] = $value ?: $this->configurations['default'][$key];

        return $this->configurations[$serviceId][$key];
    }

    /**
     * @param $serviceId
     * @return mixed
     */
    public function getDatabase($serviceId)
    {
        return $this->getConfigurationValue($serviceId, self::DATABASE);
    }
}